@@ -27,51 +27,93 @@
 //! represented, as well as the generic [`GenericColor<ComponentTy>`] type which
 //! can be used if the color space you wish to use is not represented.
 //!
+//! The [ColorInterop] trait exists to provide a "canonical" transformation to and from `cint` types.
+//! Since it is often possible to convert a color to and from multiple `cint` types, and because of
+//! how the Rust type inference system works, it can often be inconvenient to chain together `from`
+//! or `into` calls from the [From]/[Into] trait. [ColorInterop] solves this by providing a strongly
+//! typed "reference" conversion to/from `cint` types. This way, you can do things like:
+//!
+//! ```rust
+//! let color_crate1 = color_crate2.into_cint().into();
+//! // or
+//! let color_crate2 = ColorCrate2::from_cint(color_crate1.into());
+//! ```
+//!
+//! which would otherwise be quite inconvenient. **Provider crates** (those that provide their own color
+//! types) should implement the relevant [`From`]/[`Into`] implementations to and from `cint` types, and
+//! also the [ColorInterop] trait once for each color type. The [`into_cint`][ColorInterop::into_cint] and
+//! [`from_cint`][ColorInterop::from_cint] methods will then be provided automatically.
+//!
 //! ## Colors with alpha channels
 //!
-//! `cint` provides the [`Alpha<ComponentTy, ColorTy>`] and [`PremultipliedAlpha<ComponentTy, ColorTy>`]
-//! structs, which are generic over both `ComponentTy` and `ColorTy`.
+//! `cint` provides the [`Alpha<ColorTy>`] and [`PremultipliedAlpha<ColorTy>`]
+//! structs, which are generic over the inner `ColorTy`.
 //! To represent an [`EncodedSrgb<u8>`] color with a premultiplied alpha component,
-//! you'd use [`PremultipliedAlpha<u8, EncodedSrgb<u8>>`]. If, on the other hand, you want to represent
-//! an [`Oklab<f32>`] color with an independent alpha component, you'd use [`Alpha<f32, Oklab<f32>>`]
+//! you'd use [`PremultipliedAlpha<EncodedSrgb<u8>>`]. If, on the other hand, you want to represent
+//! an [`Oklab<f32>`] color with an independent alpha component, you'd use [`Alpha<Oklab<f32>>`]
 #![no_std]
 
 #[cfg(feature = "bytemuck")]
 use bytemuck::{Pod, Zeroable};
+
+/// A trait used to simpify the interface of the [`Alpha`] and [`PremultipliedAlpha`] types.
+pub trait ColorStruct {
+    type ComponentTy: Clone + Copy;
+}
+
+/// A trait that should be implemented by provider crates on their local color types so that you can call
+/// `color.to_cint()` and `Color::from_cint(cint_color)`.
+///
+/// Provider crates should also do relevant `From`/`Into` impls, but [`ColorInterop`] provides a "canonical"
+/// transformation to the closest `cint` color type.
+pub trait ColorInterop
+where
+    Self: Into<Self::CintTy>,
+{
+    type CintTy: Into<Self>;
+
+    /// Convert `self` into its canonical `cint` type.
+    fn from_cint(col: Self::CintTy) -> Self {
+        col.into()
+    }
+
+    /// Create a `Self` from its canonical `cint` type.
+    fn into_cint(self) -> Self::CintTy {
+        self.into()
+    }
+}
+
 /// A color with an alpha component.
 ///
 /// The color components and alpha component are completely separate.
 #[derive(Clone, Copy, Debug, Hash, PartialEq, PartialOrd, Eq, Ord)]
-pub struct Alpha<ComponentTy, ColorTy> {
+pub struct Alpha<ColorTy: ColorStruct> {
     /// The contained color, which is completely separate from the `alpha` value.
     pub color: ColorTy,
     /// The alpha component.
-    pub alpha: ComponentTy,
+    pub alpha: ColorTy::ComponentTy,
 }
 
 #[cfg(feature = "bytemuck")]
-unsafe impl<ComponentTy: Zeroable, ColorTy: Zeroable> Zeroable for Alpha<ComponentTy, ColorTy> {}
+unsafe impl<ColorTy: ColorStruct + Zeroable> Zeroable for Alpha<ColorTy> {}
 #[cfg(feature = "bytemuck")]
-unsafe impl<ComponentTy: Pod, ColorTy: Pod> Pod for Alpha<ComponentTy, ColorTy> {}
+unsafe impl<ColorTy: ColorStruct + Pod> Pod for Alpha<ColorTy> {}
 
 /// A premultiplied color with an alpha component.
 ///
 /// The color components have been premultiplied by the alpha component.
 #[derive(Clone, Copy, Debug, Hash, PartialEq, PartialOrd, Eq, Ord)]
-pub struct PremultipliedAlpha<ComponentTy, ColorTy> {
+pub struct PremultipliedAlpha<ColorTy: ColorStruct> {
     /// The contained color, which has been premultiplied with `alpha`
     pub color: ColorTy,
     /// The alpha component.
-    pub alpha: ComponentTy,
+    pub alpha: ColorTy::ComponentTy,
 }
 
 #[cfg(feature = "bytemuck")]
-unsafe impl<ComponentTy: Zeroable, ColorTy: Zeroable> Zeroable
-    for PremultipliedAlpha<ComponentTy, ColorTy>
-{
-}
+unsafe impl<ColorTy: ColorStruct + Zeroable> Zeroable for PremultipliedAlpha<ColorTy> {}
 #[cfg(feature = "bytemuck")]
-unsafe impl<ComponentTy: Pod, ColorTy: Pod> Pod for PremultipliedAlpha<ComponentTy, ColorTy> {}
+unsafe impl<ColorTy: ColorStruct + Pod> Pod for PremultipliedAlpha<ColorTy> {}
 
 macro_rules! color_struct {
     {
@@ -89,6 +131,10 @@ macro_rules! color_struct {
             pub $compname: ComponentTy,)+
         }
 
+        impl<CTy: Clone + Copy> ColorStruct for $name<CTy> {
+            type ComponentTy = CTy;
+        }
+
         #[cfg(feature = "bytemuck")]
         unsafe impl<ComponentTy: Zeroable> Zeroable for $name<ComponentTy> {}
         #[cfg(feature = "bytemuck")]
@@ -120,14 +166,14 @@ macro_rules! color_struct {
 
         macro_rules! impl_alpha_traits {
             ($alphaty:ident) => {
-                impl<ComponentTy> From<$alphaty<ComponentTy, $name<ComponentTy>>> for $name<ComponentTy> {
-                    fn from(col_alpha: $alphaty<ComponentTy, $name<ComponentTy>>) -> $name<ComponentTy> {
+                impl<ComponentTy: Clone + Copy> From<$alphaty<$name<ComponentTy>>> for $name<ComponentTy> {
+                    fn from(col_alpha: $alphaty<$name<ComponentTy>>) -> $name<ComponentTy> {
                         col_alpha.color
                     }
                 }
 
-                impl<ComponentTy> From<[ComponentTy; 4]> for $alphaty<ComponentTy, $name<ComponentTy>> {
-                    fn from([a, b, c, alpha]: [ComponentTy; 4]) -> $alphaty<ComponentTy, $name<ComponentTy>> {
+                impl<ComponentTy: Clone + Copy> From<[ComponentTy; 4]> for $alphaty<$name<ComponentTy>> {
+                    fn from([a, b, c, alpha]: [ComponentTy; 4]) -> $alphaty<$name<ComponentTy>> {
                         $alphaty {
                             color: $name::from([a, b, c]),
                             alpha
@@ -136,7 +182,7 @@ macro_rules! color_struct {
                 }
 
                 #[allow(clippy::from_over_into)]
-                impl<ComponentTy> Into<[ComponentTy; 4]> for $alphaty<ComponentTy, $name<ComponentTy>> {
+                impl<ComponentTy: Clone + Copy> Into<[ComponentTy; 4]> for $alphaty<$name<ComponentTy>> {
                     fn into(self) -> [ComponentTy; 4] {
                         let $alphaty {
                             color,
